/*
 * =========================================================================
 * Copyright (C) 1997 - 1998 by Visual Numerics, Inc. All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software is freely
 * granted by Visual Numerics, Inc., provided that the copyright notice
 * above and the following warranty disclaimer are preserved in human
 * readable form.
 *
 * Because this software is licenses free of charge, it is provided
 * "AS IS", with NO WARRANTY.  TO THE EXTENT PERMITTED BY LAW, VNI
 * DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO ITS PERFORMANCE, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * VNI WILL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER ARISING OUT OF THE USE
 * OF OR INABILITY TO USE THIS SOFTWARE, INCLUDING BUT NOT LIMITED TO DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, PUNITIVE, AND EXEMPLARY DAMAGES, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
 *
 * =========================================================================
 */
package com.imsl.test.math;

class TestMath extends Test
{
	void ceil()
	{
		check("ceil(0)",		Math.ceil(0.0),			0.0);
		check("ceil(-0)",		Math.ceil(NEGZERO),		NEGZERO);
		check("ceil(+inf)",		Math.ceil(POSINF),		POSINF);
		check("ceil(-inf)",		Math.ceil(NEGINF),		NEGINF);
		check("ceil(NaN)",		Math.ceil(NaN),			NaN);

		check("ceil(0.99999)",	Math.ceil(0.99999),		1.0);
		check("ceil(1)",		Math.ceil(1.0),			1.0);
		check("ceil(1.00001)",	Math.ceil(1.00001),		2.0);
		check("ceil(-0.99999)",	Math.ceil(-0.99999),	NEGZERO);
		check("ceil(-1)",		Math.ceil(-1.0),		-1.0);
		check("ceil(-1.00001)",	Math.ceil(-1.00001),	-1.0);
	}

	void floor()
	{
		check("floor(0)",		Math.floor(0.0),		0.0);
		check("floor(-0)",		Math.floor(NEGZERO),	NEGZERO);
		check("floor(inf)",		Math.floor(POSINF),		POSINF);
		check("floor(-inf)",	Math.floor(NEGINF),		NEGINF);
		check("floor(NaN)",		Math.floor(NaN),		NaN);

		check("floor(0.99999)",	Math.floor(0.99999),	0.0);
		check("floor(1)",		Math.floor(1.0),		1.0);
		check("floor(1.00001)",	Math.floor(1.00001),	1.0);
		check("floor(-0.99999)",Math.floor(-0.99999),	-1.0);
		check("floor(-1)",		Math.floor(-1.0),		-1.0);
		check("floor(-1.00001)",Math.floor(-1.00001),	-2.0);
	}
	
	void round()
	{
		check("round(0)",		Math.round(0.0),		0.0);
		check("round(-0)",		Math.round(NEGZERO),	0.0);
		check("round(+inf)",	Math.round(POSINF),		Long.MAX_VALUE);
		check("round(-inf)",	Math.round(NEGINF),		Long.MIN_VALUE);
		check("round(NaN)",		Math.round(NaN),		0);

		check("round(0.99999)",	Math.round(0.99999),	1.0);
		check("round(1)",		Math.round(1.0),		1.0);
		check("round(1.00001)",	Math.round(1.00001),	1.0);
		check("round(-0.99999)",Math.round(-0.99999),	-1.0);
		check("round(-1)",		Math.round(-1.0),		-1.0);
		check("round(-1.00001)",Math.round(-1.00001),	-1.0);
	}

	void rint()
	{
		check("rint(0)",		Math.rint(0.0),			0.0);
		check("rint(-0)",		Math.rint(NEGZERO),		NEGZERO);
		check("rint(+inf)",		Math.rint(POSINF),		POSINF);
		check("rint(-inf)",		Math.rint(NEGINF),		NEGINF);
		check("rint(NaN)",		Math.rint(NaN),			NaN);
		
		check("rint(0.99999)",	Math.rint(0.99999),		1.0);
		check("rint(1)",		Math.rint(1.0),			1.0);
		check("rint(1.00001)",	Math.rint(1.00001),		1.0);
		check("rint(-0.99999)",	Math.rint(-0.99999),	-1.0);
		check("rint(-1)",		Math.rint(-1.0),		-1.0);
		check("rint(-1.00001)",	Math.rint(-1.00001),	-1.0);
	}

	void abs()
	{
		check("abs(0)",			Math.abs(0),		0);
		check("abs(-0)",		Math.abs(NEGZERO),	0);
		check("abs(inf)",		Math.abs(POSINF),	POSINF);
		check("abs(-inf)",		Math.abs(NEGINF),	POSINF);
		check("abs(NaN)",		Math.abs(NaN),		NaN);
		
		check("abs(16)",		Math.abs(16),		16);
		check("abs(1e300)",		Math.abs(1e300),	1e300);
		check("abs(-16)",		Math.abs(-16),		16);
		check("abs(-1e300)",	Math.abs(-1e300),	1e300);
	}
	
	void sqrt()
	{
		check("sqrt(0)",		Math.sqrt(0),		0);
		check("sqrt(-0)",		Math.sqrt(NEGZERO), NEGZERO);
		check("sqrt(-1)",		Math.sqrt(-1),		NaN);
		check("sqrt(+inf)",		Math.sqrt(POSINF),	POSINF);
		check("sqrt(-inf)",		Math.sqrt(NEGINF),	NaN);
		check("sqrt(NaN)",		Math.sqrt(NaN),		NaN);		

		check("sqrt(16)",		Math.sqrt(16),		4);
		check("sqrt(1e300)",	Math.sqrt(1e300),	1e150);
	}
	
	void exp()
	{
		check("exp(0)",			Math.exp(0),		1);
		check("exp(-0)",		Math.exp(NEGZERO),	1);
		check("exp(inf)",		Math.exp(POSINF),	POSINF);
		check("exp(-inf)",		Math.exp(NEGINF),	0);
		check("exp(NaN)",		Math.exp(NaN),		NaN);

		check("exp(2.3)",		Math.exp(2.3),		9.974182454814718);
		check("exp(2.3e-4)",	Math.exp(2.3e-4),	1.0002300264520279);
		check("exp(2.3e2)",		Math.exp(2.3e2),	7.722018499983836e99);
	}

	void log()
	{
		check("log(0)",			Math.log(0),		NEGINF);
		check("log(-0)",		Math.log(NEGZERO),	NEGINF);
		check("log(1)",			Math.log(1),		0);
		check("log(-1)",		Math.log(-1),		NaN);
		check("log(inf)",		Math.log(POSINF),	POSINF);
		check("log(NaN)",		Math.log(NaN),		NaN);
		
		check("log(100)",		Math.log(100),		2*Math.log(10));
		check("log(1e300)",		Math.log(1e300),	300*Math.log(10));
		check("log(-inf)",		Math.log(NEGINF),	NaN);
	}

	void sin()
	{
		check("sin(0)",			Math.sin(0),		0);
		check("sin(-0)",		Math.sin(NEGZERO),	NEGZERO);
		check("sin(+inf)",		Math.sin(POSINF),	NaN);
		check("sin(-inf)",		Math.sin(NEGINF),	NaN);
		check("sin(NaN)",		Math.sin(NaN),		NaN);

		check("sin(100*PI)"	,	Math.sin(100*PI),	0);
		check("sin(1e300*PI)",	Math.sin(1e300*PI), -0.902464352009667);
	}
	
	void cos()
	{
		check("cos(0)",			Math.cos(0),		1);
		check("cos(-0)",		Math.cos(NEGZERO),	1);
		check("cos(+inf)",		Math.cos(POSINF),	NaN);
		check("cos(-inf)",		Math.cos(NEGINF),	NaN);
		check("cos(NaN)",		Math.cos(NaN),		NaN);

		check("cos(100*PI)"	,	Math.cos(100*PI),	1);
		check("cos(1e300*PI)",	Math.cos(1e300*PI),-0.43076454514243845);
	}

	void tan()
	{
		check("tan(0)",			Math.tan(0),		0);
		check("tan(-0)",		Math.tan(NEGZERO),	NEGZERO);
		check("tan(inf)",		Math.tan(POSINF),	NaN);
		check("tan(-inf)",		Math.tan(NEGINF),	NaN);
		check("tan(NaN)",		Math.tan(NaN),		NaN);

		check("tan(100*PI)"	,	Math.tan(100*PI),	0);
		check("tan(1e300*PI)",	Math.tan(1e300*PI),2.0950293198138072);
	}

	void asin()
	{
		check("asin(0)",		Math.asin(0),		0);
		check("asin(-0)",		Math.asin(NEGZERO),	NEGZERO);
		check("asin(1)",		Math.asin(1),		PI/2);
		check("asin(-1)",		Math.asin(-1),		-PI/2);
		check("asin(2)",		Math.asin(2),		NaN);
		check("asin(+inf)",		Math.asin(POSINF),	NaN);
		check("asin(-inf)",		Math.asin(NEGINF),	NaN);
		check("asin(NaN)",		Math.asin(NaN),		NaN);
	}
	
	void acos()
	{
		check("acos(0)",		Math.acos(0),		PI/2);
		check("acos(1)",		Math.acos(1),		0);
		check("acos(2)",		Math.acos(2),		NaN);
		check("acos(+inf)",		Math.acos(POSINF),	NaN);
		check("acos(-inf)",		Math.acos(NEGINF),	NaN);
		check("acos(-0)",		Math.acos(NEGZERO),	PI/2);
		check("acos(NaN)",		Math.acos(NaN),		NaN);
	}

	void atan()
	{
		check("atan(0)",		Math.atan(0),		0);
		check("atan(-0)",		Math.atan(NEGZERO),	NEGZERO);
		check("atan(1)",		Math.atan(1),		PI/4);
		check("atan(-1)",		Math.atan(-1),		-PI/4);
		check("atan(+inf)",		Math.atan(POSINF),	PI/2);
		check("atan(-inf)",		Math.atan(NEGINF),	-PI/2);
		check("atan(NaN)",		Math.atan(NaN),		NaN);
	}

	void min()
	{
		check("min(0,0)",			Math.min(0,0),			0);	
		check("min(0,1)",			Math.min(0,1),			0);
		check("min(1,0)",			Math.min(1,0),			0);
		check("min(1,1)",			Math.min(1,1),			1);

		check("min(0,-1)",			Math.min(0,-1),			-1);
		check("min(-0,-1)",			Math.min(NEGZERO,-1),	-1);
		check("min(-1,0)",			Math.min(-1,0),			-1);
		check("min(-1,-1)",			Math.min(-1,-1),		-1);

		check("min(-0,0)",			Math.min(NEGZERO,0),	NEGZERO);
		check("min(-0,1)",			Math.min(NEGZERO,1),	NEGZERO);
		check("min(0,-0)",			Math.min(0, NEGZERO),	NEGZERO);
		check("min(0,-1)",			Math.min(1, NEGZERO),	NEGZERO);
		
		check("min(0,+inf)",		Math.min(0,POSINF),		0);
		check("min(0,-inf)",		Math.min(0,NEGINF),		NEGINF);
		check("min(1,+inf)",		Math.min(1,POSINF),		1);
		check("min(1,-inf)",		Math.min(1,NEGINF),		NEGINF);

		check("min(+inf,0)",		Math.min(POSINF,0),		0);
		check("min(-inf,0)",		Math.min(NEGINF,0),		NEGINF);
		check("min(+inf,1)",		Math.min(POSINF,1),		1);
		check("min(-inf,1)",		Math.min(NEGINF,1),		NEGINF);

		check("min(+inf,+inf)",		Math.min(POSINF,POSINF),	POSINF);
		check("min(+inf,-inf)",		Math.min(POSINF,NEGINF),	NEGINF);
		check("min(-inf,+inf)",		Math.min(NEGINF,POSINF),	NEGINF);
		check("min(-inf,-inf)",		Math.min(NEGINF,NEGINF),	NEGINF);
		
		check("min(0,NaN)",			Math.min(0,NaN),		NaN);
		check("min(NaN,0)",			Math.min(NaN,0),		NaN);
		check("min(1,NaN)",			Math.min(1,NaN),		NaN);
		check("min(NaN,1)",			Math.min(NaN,1),		NaN);
		check("min(+inf,NaN)",		Math.min(POSINF,NaN),	NaN);
		check("min(NaN,+inf)",		Math.min(NaN,POSINF),	NaN);
	}

	void max()
	{
		check("max(0,0)",			Math.max(0,0),			0);	
		check("max(0,1)",			Math.max(0,1),			1);
		check("max(1,0)",			Math.max(1,0),			1);
		check("max(1,1)",			Math.max(1,1),			1);

		check("max(0,-1)",			Math.max(0,-1),			0);
		check("max(-0,-1)",			Math.max(NEGZERO,-1),	NEGZERO);
		check("max(-1,0)",			Math.max(-1,0),			0);
		check("max(-1,-1)",			Math.max(-1,-1),		-1);

		check("max(-0,0)",			Math.max(NEGZERO,0),	0);
		check("max(-0,1)",			Math.max(NEGZERO,1),	1);
		check("max(0,-0)",			Math.max(0,NEGZERO),	0);
		check("max(0,-1)",			Math.max(1,NEGZERO),	1);
		
		check("max(0,+inf)",		Math.max(0,POSINF),		POSINF);
		check("max(0,-inf)",		Math.max(0,NEGINF),		0);
		check("max(1,+inf)",		Math.max(1,POSINF),		POSINF);
		check("max(1,-inf)",		Math.max(1,NEGINF),		1);

		check("max(+inf,0)",		Math.max(POSINF,0),		POSINF);
		check("max(-inf,0)",		Math.max(NEGINF,0),		0);
		check("max(+inf,1)",		Math.max(POSINF,1),		POSINF);
		check("max(-inf,1)",		Math.max(NEGINF,1),		1);

		check("max(+inf,+inf)",		Math.max(POSINF,POSINF),POSINF);
		check("max(+inf,-inf)",		Math.max(POSINF,NEGINF),POSINF);
		check("max(-inf,+inf)",		Math.max(NEGINF,POSINF),POSINF);
		check("max(-inf,-inf)",		Math.max(NEGINF,NEGINF),NEGINF);
		
		check("max(0,NaN)",			Math.max(0,NaN),		NaN);
		check("max(NaN,0)",			Math.max(NaN,0),		NaN);
		check("max(1,NaN)",			Math.max(1,NaN),		NaN);
		check("max(NaN,1)",			Math.max(NaN,1),		NaN);
		check("max(+inf,NaN)",		Math.max(POSINF,NaN),	NaN);
		check("max(NaN,+inf)",		Math.max(NaN,POSINF),	NaN);
	}

	void atan2()
	{
		check("atan2(0,1)",			Math.atan2(0,1),		0);
		check("atan2(-0,1)",		Math.atan2(NEGZERO,1),	NEGZERO);
		
		check("atan2(0,0)",			Math.atan2(0,0),		0);
		check("atan2(-0,0)",		Math.atan2(NEGZERO,0),	NEGZERO);

		check("atan2(0,-0)",		Math.atan2(0,NEGZERO),	PI);
		check("atan2(-0,-0)",		Math.atan2(NEGZERO,NEGZERO),-PI);

		check("atan2(1,0)",			Math.atan2(1,0),		PI/2);
		check("atan2(1,-0)",		Math.atan2(1,NEGZERO),	PI/2);
		
		check("atan2(-1,0)",		Math.atan2(-1,0),		-PI/2);
		check("atan2(-1,-0)",		Math.atan2(-1,NEGZERO),	-PI/2);

		check("atan2(1,+inf)",		Math.atan2(1,POSINF),	0);
		check("atan2(-1,+inf)",		Math.atan2(-1,POSINF),	NEGZERO);

		check("atan2(+inf,1)",		Math.atan2(POSINF,1),	PI/2);
		check("atan2(-inf,1)",		Math.atan2(NEGINF,1),	-PI/2);
		
		check("atan2(1,-inf)",		Math.atan2(1,NEGINF),	PI);
		check("atan2(-1,-inf)",		Math.atan2(-1,NEGINF),	-PI);

		check("atan2(+inf,+inf)",	Math.atan2(POSINF,POSINF),	PI/4);
		check("atan2(-inf,+inf)",	Math.atan2(NEGINF,POSINF),	-PI/4);

		check("atan2(+inf,-inf)",	Math.atan2(POSINF,NEGINF),	3*PI/4);
		check("atan2(-inf,-inf)",	Math.atan2(NEGINF,NEGINF),	-3*PI/4);
		
		check("atan2(1,1)",			Math.atan2(1,1),		PI/4);

		check("atan2(0,-1)",		Math.atan2(0,-1),		PI);
		check("atan2(-1,0)",		Math.atan2(-1,0),		-PI/2);
		check("atan2(-1,-1)",		Math.atan2(-1,-1),		-0.75*PI);

		check("atan2(1e300,1e300)",		Math.atan2(1e300,1e300),	PI/4);
		check("atan2(1e300,-1e300)",	Math.atan2(1e300,-1e300),	3*PI/4);
		check("atan2(-1e300,1e300)",	Math.atan2(-1e300,1e300),	-PI/4);
		check("atan2(-1e300,-1e300)",	Math.atan2(-1e300,-1e300),	-3*PI/4);
		
		check("atan2(0,+inf)",		Math.atan2(0,POSINF),	0);
		check("atan2(0,-inf)",		Math.atan2(0,NEGINF),	PI);

		check("atan2(+inf,0)",		Math.atan2(POSINF,0),	PI/2);
		check("atan2(-inf,0)",		Math.atan2(NEGINF,0),	-PI/2);

		check("atan2(-inf,-inf)",	Math.atan2(NEGINF,NEGINF),	-3*PI/4);
		
		check("atan2(0,NaN)",		Math.atan2(0,NaN),		NaN);
		check("atan2(NaN,0)",		Math.atan2(NaN,0),		NaN);
		check("atan2(1,NaN)",		Math.atan2(1,NaN),		NaN);
		check("atan2(NaN,1)",		Math.atan2(NaN,1),		NaN);
		check("atan2(+inf,NaN)",	Math.atan2(POSINF,NaN),	NaN);
		check("atan2(NaN,+inf)",	Math.atan2(NaN,POSINF),	NaN);
	}

	void pow()
	{
		check("pow(0,0)",			Math.pow(0,0),			1);
		check("pow(-0,0)",			Math.pow(NEGZERO,0),	1);
		check("pow(1,0)",			Math.pow(1,0),			1);
		check("pow(-1,0)",			Math.pow(-1,0),			1);
		check("pow(+inf,0)",		Math.pow(POSINF,0),		1);
		check("pow(-inf,0)",		Math.pow(NEGINF,0),		1);
		check("pow(NaN,0)",			Math.pow(NaN,0),		1);

		check("pow(1.1,+inf)",		Math.pow(1.1,POSINF),	POSINF);
		check("pow(0.9,+inf)",		Math.pow(0.9,POSINF),	0);
		check("pow(0,+inf)",		Math.pow(0,POSINF),		0);
		check("pow(-1.1,+inf)",		Math.pow(-1.1,POSINF),	POSINF);
		check("pow(-0.9,+inf)",		Math.pow(-0.9,POSINF),	0);

		check("pow(1.1,-inf)",		Math.pow(1.1,NEGINF),	0);
		check("pow(0.9,-inf)",		Math.pow(0.9,NEGINF),	POSINF);
		check("pow(0,-inf)",		Math.pow(0,NEGINF),		POSINF);
		check("pow(-1.1,-inf)",		Math.pow(-1.1,NEGINF),	0);
		check("pow(-0.9,-inf)",		Math.pow(-0.9,NEGINF),	POSINF);

		check("pow(+inf,1)",		Math.pow(POSINF,1),		POSINF);
		check("pow(+inf,+inf)",		Math.pow(POSINF,POSINF),POSINF);
		check("pow(+inf,-1)",		Math.pow(POSINF,-1),	0);
		check("pow(+inf,-inf)",		Math.pow(POSINF,NEGINF),0);

		check("pow(-inf,1)",		Math.pow(NEGINF,1.0),	NEGINF);
		check("pow(-inf,3)",		Math.pow(NEGINF,3.0),	NEGINF);
		check("pow(-inf,2)",		Math.pow(NEGINF,2.0),	POSINF);
		check("pow(-inf,2.5)",		Math.pow(NEGINF,2.5),	POSINF);
		check("pow(-inf,+inf)",		Math.pow(NEGINF,POSINF),POSINF);
		check("pow(-inf,-3)",		Math.pow(NEGINF,-3),	NEGZERO);
		check("pow(-inf,-2.5)",		Math.pow(NEGINF,-2.5),	0);
		check("pow(-inf,-inf)",		Math.pow(NEGINF,NEGINF),0);

		check("pow(0,NaN)",			Math.pow(0,NaN),		NaN);
		check("pow(1,NaN)",			Math.pow(1,NaN),		NaN);
		check("pow(+inf,NaN)",		Math.pow(POSINF,NaN),	NaN);
		check("pow(-inf,NaN)",		Math.pow(NEGINF,NaN),	NaN);
		
		
		check("pow(NaN,1)",			Math.pow(NaN,1),		NaN);
		check("pow(NaN,+inf)",		Math.pow(NaN,POSINF),	NaN);
		
		check("pow(1,+inf)",		Math.pow(1,POSINF),		NaN);
		check("pow(-1,+inf)",		Math.pow(-1,POSINF),	NaN);
		check("pow(1,-inf)",		Math.pow(1,NEGINF),		NaN);
		check("pow(-1,-inf)",		Math.pow(-1,NEGINF),	NaN);

		check("pow(-2,2.5)",		Math.pow(-2,2.5),		NaN);
		
		check("pow(0,-3)",			Math.pow(0,-3),			POSINF);
		check("pow(-0,-3)",			Math.pow(NEGZERO,-3),	NEGINF);

		check("pow(0,-2.5)",		Math.pow(0,-2.5),		POSINF);
		check("pow(-0,-2.5)",		Math.pow(NEGZERO,-2.5),	POSINF);

		check("pow(0,3)",			Math.pow(0,3),			0);
		check("pow(-0,3)",			Math.pow(NEGZERO,3),	NEGZERO);
		
		check("pow(0,2.5)",			Math.pow(0,2.5),		0);
		check("pow(-0,2.5)",		Math.pow(NEGZERO,2.5),	0);


		check("pow(1,1)",			Math.pow(1,1),			1);
		check("pow(0,-1)",			Math.pow(0,-1),			POSINF);
		check("pow(-0,-1)",			Math.pow(NEGZERO,-1),	NEGINF);
		check("pow(-1,-1)",			Math.pow(-1,-1),		-1);
		check("pow(-0,1)",			Math.pow(NEGZERO,1),	NEGZERO);
		check("pow(-0,2)",			Math.pow(NEGZERO,2),	0);
	}

	void IEEEremainder()
	{
		check("IEEEremainder(0,0)",		Math.IEEEremainder(0,0),			NaN);
		check("IEEEremainder(0,-0)",	Math.IEEEremainder(0,NEGZERO),		NaN);
		check("IEEEremainder(-0,0)",	Math.IEEEremainder(NEGZERO,0),		NaN);
		check("IEEEremainder(-0,-0)",	Math.IEEEremainder(NEGZERO,NEGZERO),NaN);

		check("IEEEremainder(1,0)",		Math.IEEEremainder(1,0),			NaN);
		check("IEEEremainder(-1,0)",	Math.IEEEremainder(-1,0),			NaN);
		check("IEEEremainder(1,-0)",	Math.IEEEremainder(1,NEGZERO),		NaN);
		check("IEEEremainder(-1,-0)",	Math.IEEEremainder(-1,NEGZERO),		NaN);
		
		check("IEEEremainder(0,1)",		Math.IEEEremainder(0,1),			0);
		check("IEEEremainder(0,-1)",	Math.IEEEremainder(0,-1),			0);
		
		check("IEEEremainder(-0,1)",	Math.IEEEremainder(NEGZERO,1),		NEGZERO);
		check("IEEEremainder(-0,-1)",	Math.IEEEremainder(NEGZERO,-1),		NEGZERO);

		check("IEEEremainder(5,1)",		Math.IEEEremainder(5,1),			0);
		check("IEEEremainder(5,-1)",	Math.IEEEremainder(5,-1),			0);

		check("IEEEremainder(-5,1)",	Math.IEEEremainder(-5,1),			NEGZERO);
		check("IEEEremainder(-5,-1)",	Math.IEEEremainder(-5,-1),			NEGZERO);

		
		check("IEEEremainder(+inf,+inf)",	Math.IEEEremainder(POSINF,POSINF),	NaN);
		check("IEEEremainder(+inf,-inf)",	Math.IEEEremainder(POSINF,NEGINF),	NaN);
		check("IEEEremainder(-inf,+inf)",	Math.IEEEremainder(NEGINF,POSINF),	NaN);
		check("IEEEremainder(-inf,-inf)",	Math.IEEEremainder(NEGINF,NEGINF),	NaN);

		check("IEEEremainder(+inf,0)",		Math.IEEEremainder(POSINF,0),		NaN);
		check("IEEEremainder(-inf,0)",		Math.IEEEremainder(NEGINF,0),		NaN);
		check("IEEEremainder(+inf,1)",		Math.IEEEremainder(POSINF,1),		NaN);
		check("IEEEremainder(-inf,1)",		Math.IEEEremainder(NEGINF,1),		NaN);
		check("IEEEremainder(+inf,-1)",		Math.IEEEremainder(POSINF,-1),		NaN);
		check("IEEEremainder(-inf,-1)",		Math.IEEEremainder(NEGINF,-1),		NaN);

		check("IEEEremainder(0,+inf)",		Math.IEEEremainder(0,POSINF),		0);
		check("IEEEremainder(0,-inf)",		Math.IEEEremainder(0,NEGINF),		0);
		check("IEEEremainder(1,+inf)",		Math.IEEEremainder(1,POSINF),		1);
		check("IEEEremainder(1,-inf)",		Math.IEEEremainder(1,NEGINF),		1);
		check("IEEEremainder(-1,inf)",		Math.IEEEremainder(-1,POSINF),		-1);
		check("IEEEremainder(-1,-inf)",		Math.IEEEremainder(-1,NEGINF),		-1);

		check("IEEEremainder(0,NaN)",		Math.IEEEremainder(0,NaN),			NaN);
		check("IEEEremainder(NaN,0)",		Math.IEEEremainder(NaN,0),			NaN);
		check("IEEEremainder(1,NaN)",		Math.IEEEremainder(1,NaN),			NaN);
		check("IEEEremainder(NaN,1)",		Math.IEEEremainder(NaN,1),			NaN);
		check("IEEEremainder(+inf,NaN)",	Math.IEEEremainder(POSINF,NaN),		NaN);
		check("IEEEremainder(NaN,+inf)",	Math.IEEEremainder(NaN,POSINF),		NaN);
	}

	protected String getTestName()
	{
		return "java.lang.Math";
	}

	public void all()
	{
		ceil();
		floor();
		round();
		rint();
		abs();
		sqrt();
		exp();
		log();
		sin();
		cos();
		tan();
		asin();
		acos();
		atan();
		min();
		max();
		atan2();
		pow();
		IEEEremainder();
	}
}
