/*
 * =========================================================================
 * Copyright (C) 1997 - 1998 by Visual Numerics, Inc. All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software is freely
 * granted by Visual Numerics, Inc., provided that the copyright notice
 * above and the following warranty disclaimer are preserved in human
 * readable form.
 *
 * Because this software is licenses free of charge, it is provided
 * "AS IS", with NO WARRANTY.  TO THE EXTENT PERMITTED BY LAW, VNI
 * DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO ITS PERFORMANCE, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * VNI WILL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER ARISING OUT OF THE USE
 * OF OR INABILITY TO USE THIS SOFTWARE, INCLUDING BUT NOT LIMITED TO DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, PUNITIVE, AND EXEMPLARY DAMAGES, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
 *
 * =========================================================================
 */

package com.imsl.test.math;
import com.imsl.math.Sfun;

class TestSfun extends Test
{
	void sinh()
	{
		// C9X test cases
		check(
			"sinh(0.0)",
			Sfun.sinh(0.0),
			0);
		check(
			"sinh(-0)",
			Sfun.sinh(NEGZERO),
			NEGZERO);
		check(
			"sinh(+inf)",
			Sfun.sinh(POSINF),
			POSINF);
		check(
			"sinh(-inf)",
			Sfun.sinh(NEGINF),
			NEGINF);
		check(
			"sinh(NaN)",
			Sfun.sinh(NaN),
			NaN);

		// IMSL test cases
		check(
			"sinh(1e-15)",
			Sfun.sinh(1e-15),
			1e-15);
		check(
			"sinh(0.5)",
			Sfun.sinh(0.5),
			0.5210953054937473616224256264114915591059);
		check(
			"sinh(-1)",
			Sfun.sinh(-1),
			-1.175201193643801456882381850595600815156);
		check(
			"sinh(2)",
			Sfun.sinh(2),
			3.626860407847018767668213982801261704886);
		check(
			"sinh(35)",
			Sfun.sinh(35),
			793006726156715.3640648223128870148067501);

	}
	
	void cosh()
	{
		// C9X test cases
		check(
			"cosh(0.0)",
			Sfun.cosh(0.0),
			1);
		check("cosh(-0)",
			Sfun.cosh(NEGZERO),
			1);
		check("cosh(+inf)",
			Sfun.cosh(POSINF),	POSINF);
		check("cosh(-inf)",
			Sfun.cosh(NEGINF),
			NEGINF);
		check("cosh(NaN)",
			Sfun.cosh(NaN),	
			NaN);
		// IMSL test cases
		check(
			"cosh(-5)",
			Sfun.cosh(-5),
			74.20994852478784444410610804448771402387);
		check(
			"cosh(5)",
			Sfun.cosh(5),
			74.20994852478784444410610804448771402387);
		check(
			"cosh(10)",
			Sfun.cosh(10),
			11013.23292010332313972137609043787996345);
		check(
			"cosh(35)",
			Sfun.cosh(35),
			793006726156715.364064822312887645318426);
	}

	void tanh()
	{
		// C9X test cases
		check(
			"tanh(0.0)",
			Sfun.tanh(0.0),
			0);
		check(
			"tanh(-0)",
			Sfun.tanh(NEGZERO),
			NEGZERO);
		check(
			"tanh(+inf)",
			Sfun.tanh(POSINF),
			1);
		check(
			"tanh(-inf)",
			Sfun.tanh(NEGINF),
			-1);
		check(
			"tanh(NaN)",
			Sfun.tanh(NaN),
			NaN);
	}

	void asinh()
	{
		// C9X test cases
		check(
			"asinh(0.0)",
			Sfun.asinh(0.0),
			0.0);
		check("asinh(-0)",
			Sfun.asinh(NEGZERO),
			NEGZERO);
		check("asinh(+inf)",
			Sfun.asinh(POSINF),
			POSINF);
		check("asinh(-inf)",
			Sfun.asinh(NEGINF),
			NEGINF);
		check("asinh(NaN)",
			Sfun.asinh(NaN),
			NaN);

		// IMSL test cases
		check(
			"asinh(1.0e-15)",
			Sfun.asinh(1.0e-15),
			1.0e-15);
		check(
			"asinh(0.5)",
			Sfun.asinh(0.5),
			0.4812118250596034474977589134243684231352);
		check(
			"asinh(1)",
			Sfun.asinh(1),
			0.8813735870195430252326093249797923090282);
		check(
			"asinh(2)",
			Sfun.asinh(2),
			1.443635475178810342493276740273105269406);
		check(
			"asinh(1.0e+15)",
			Sfun.asinh(1.0e+15),
			35.23192357547063056968710394172388968209);

		// Additional tests
		check("asinh(0.99999)",
			Sfun.sinh(Sfun.asinh(0.99999)),
			0.99999);
		check("asinh(1.00001)",
			Sfun.sinh(Sfun.asinh(1.00001)),
			1.00001);
		check("asinh(1.0e40)",
			Sfun.sinh(Sfun.asinh(1.0e40)),
			1.0e40);
	}
	
	void acosh()
	{
		// C9X test cases
		check(
			"acosh(1)",
			Sfun.acosh(1.0),
			0.0);
		check(
			"acosh(+inf)",
			Sfun.acosh(POSINF),
			POSINF);
		check("acosh(-inf)",
			Sfun.acosh(NEGINF),
			NaN);
		check("acosh(NaN)",
			Sfun.acosh(NaN),
			NaN);

		// IMSL test cases
		check(
			"acosh(1)",
			Sfun.acosh(1.0),
			0.0);
		check(
			"acosh(1.0e+15)",
			Sfun.acosh(1.0e+15),
			35.23192357547063056968710394172338968209);
		check(
			"acosh(1.0e+30)",
			Sfun.acosh(1.0e+30),
			69.77069997038131582995697576198910279611);
		check(
			"acosh(1-eps)",
			Sfun.acosh(1.0-Sfun.EPSILON_SMALL),
			NaN);

		// Additional test cases
		check(
			"acosh(0.99999)",
			Sfun.acosh(0.99999),
			NaN);
		check(
			"acosh(1.00001)",
			Sfun.cosh(Sfun.acosh(1.00001)),
			1.00001);
		check(
			"acosh(-0.99999)",
			Sfun.acosh(-0.99999),
			NaN);
		check(
			"acosh(1.0e40)",
			Sfun.cosh(Sfun.acosh(1.0e40)),
			1.0e40);
	}

	void atanh()
	{
		// C9X test cases
		check(
			"atanh(0.0)",
			Sfun.atanh(0.0),
			0.0);
		check(
			"atanh(-0)",
			Sfun.atanh(NEGZERO),
			NEGZERO);
		check(
			"atanh(+1)",
			Sfun.atanh(1.0),
			POSINF);
		check(
			"atanh(-1)",
			Sfun.atanh(-1.0),
			NEGINF);
		check(
			"atanh(+inf)",
			Sfun.atanh(POSINF),
			NaN);
		check(
			"atanh(-inf)",
			Sfun.atanh(NEGINF),
			NaN);
		check(
			"atanh(NaN)",
			Sfun.atanh(NaN),
			NaN);

		// IMSL test cases
		check(
			"atanh(-0.25)",
			Sfun.atanh(-0.25),
			-0.2554128118829953416027570481518309674391);
		check(
			"atanh(0)",
			Sfun.atanh(0),
			0);
		check(
			"atanh(1.0e-15)",
			Sfun.atanh(1.0e-15),
			1.0e-15);
		check(
			"atanh(0.25)",
			Sfun.atanh(0.25),
			0.2554128118829953416027570481518309674391);
		check(
			"atanh(0.5)",
			Sfun.atanh(0.5),
			0.549306144334054845697622618461262852323);
		check(
			"atanh(0.75)",
			Sfun.atanh(0.75),
			0.9729550745276566525526763717215898648185);

		// Additional test cases
		check(
			"atanh(1.0e-7)",
			Sfun.tanh(Sfun.atanh(1.0e-7)),
			1.0e-7);
		check(
			"atanh(0.99999)",
			Sfun.tanh(Sfun.atanh(0.99999)),
			0.99999);
		check(
			"atanh(1.00001)",
			Sfun.atanh(1.00001),
			NaN);
		check(
			"atanh(1.0e40)",
			Sfun.atanh(1.0e40),
			NaN);
	}

	void fact()
	{
		check(
			"fact(0)",
			Sfun.fact(0),
			1);
		check("fact(2)",
			Sfun.fact(2),
			2);
		check("fact(5)",
			Sfun.fact(5),
			5*4*3*2*1);
		check("fact(171)",
			Sfun.fact(171),
			POSINF);
		check("fact(-2)",
			Sfun.fact(-2),
			NaN);
	}

	void gamma()
	{
		// IMSL test cases
		check(
			"gamma(-4.5)",
			Sfun.gamma(-4.5),
			-0.06001960130050424642702789361578481042277);
		check(
			"gamma(4)",
			Sfun.gamma(4),
			6);
		check(
			"gamma(15)",
			Sfun.gamma(15),
			87178291200.0);
		check(
			"gamma(0)",
			Sfun.gamma(0),
			NaN);
		check(
			"gamma(-2)",
			Sfun.gamma(-2),
			NaN);

		// Additional test cases
		check(
			"gamma(0)",
			Sfun.gamma(0),
			NaN);
		check(
			"gamma(2)",
			Sfun.gamma(2),
			1);
		check(
			"gamma(5)",
			Sfun.gamma(5),
			4*3*2*1);
		check(
			"gamma(1.5)",
			Sfun.gamma(1.5),
			0.5*Math.sqrt(Math.PI));
		check(
			"gamma(101)",
			Sfun.gamma(101),
			9.3326215443944152682e157); // A&S Table 6.6
		check(
			"gamma(172)",
			Sfun.gamma(172),
			POSINF);
		check(
			"gamma(-2)",
			Sfun.gamma(-2),
			NaN);
		check(
			"gamma(-20)",
			Sfun.gamma(-20),
			NaN);
		check(
			"gamma(-2.5)",
			Sfun.gamma(-2.5),
			-Math.PI/Sfun.gamma(3.5));
	}

	void logGamma()
	{
		check(
			"logGamma(0)",
			Sfun.logGamma(0),
			NaN);
		check(
			"logGamma(2)",
			Sfun.logGamma(2),
			0);
		check(
			"logGamma(5)",
			Sfun.logGamma(5),
			Math.log(4)+Math.log(3)+Math.log(2));
		check(
			"logGamma(1.5)",
			Sfun.logGamma(1.5),
			Math.log(0.5)+0.5*Math.log(Math.PI));
		check(
			"logGamma(101)",
			Sfun.logGamma(101),
			Math.log(9.3326215443944152682)+157*Math.log(10)); // A&S Table 6.6
		//check("logGamma(172)",	Sfun.logGamma(172),	POSINF);
		check(
			"logGamma(-2)",
			Sfun.logGamma(-2),
			NaN);
		check(
			"logGamma(-20)",
			Sfun.logGamma(-20),
			NaN);
		check(
			"logGamma(-2.5)",
			Sfun.logGamma(-2.5),
			Math.log(Math.PI)-Sfun.logGamma(3.5));
	}

	void erf()
	{
		// IMSL test cases
		check(
			"erf(-2)",
			Sfun.erf(-2),
			-0.9953222650189527341620692563672529286109);
		check(
			"erf(-1)",
			Sfun.erf(-1),
			-0.8427007929497148693412206350826092592961);
		check(
			"erf(1e-15)",
			Sfun.erf(1e-15),
			0.1128379167095512573896158903121169045299e-14);
		check(
			"erf(0.5)",
			Sfun.erf(0.5),
			0.5204998778130465376827466538919645287365);
		check(
			"erf(1)",
			Sfun.erf(1),
			0.8427007929497148693412206350826092592961);
		check(
			"erf(2)",
			Sfun.erf(2),
			0.9953222650189527341620692563672529286109);
	}

	void erfc()
	{
		// IMSL test cases
		check(
			"erfc(-2)",
			Sfun.erfc(-2),
			 1.995322265018952734162069256367252928611);
		check(
			"erfc(-0.9)",
			Sfun.erfc(-0.9),
			2-Sfun.erfc(0.9));
		check(
			"erfc(-1)",
			Sfun.erfc(-1),
			1.842700792949714869341220635082609259296);
		check(
			"erfc(1e-15)",
			Sfun.erfc(1e-15),
			0.9999999999999988716208329044874261038411);
		check(
			"erfc(0.5)",
			Sfun.erfc(0.5),
			0.4795001221869534623172533461080354712635);
		check(
			"erfc(1)",
			Sfun.erfc(1),
			0.1572992070502851306587793649173907407039);
		check(
			"erfc(2)",
			Sfun.erfc(2),
			0.0046777349810472658379307436327470713891);
		check(
			"erfc(4)",
			Sfun.erfc(4),
			0.154172579002800188521596734868840e-7);
		check(
			"erfc(6)",
			Sfun.erfc(6),
			 2.15197367124989131165933504e-17);
		
		// Additional test cases
		check(
			"erfc(-12)",
			Sfun.erfc(-12),
			2);

	}

	protected String getTestName()
	{
		return "com.imsl.math.Sfun";
	}
	
	public void all()
	{
		//cot();
		sinh();
		cosh();
		tanh();
		asinh();
		acosh();
		atanh();
		fact();
		gamma();
		logGamma();
		//logBeta();
		erf();
		erfc();
	}
}
